==> llmswap-2.1.4.tar.gz: llmswap-2.1.4/examples/smart_cost_optimizer.py <==
"""
Smart Cost Optimizer - Save 50-90% on LLM API costs

This example shows how llmswap automatically optimizes costs by:
1. Using cache for repeated queries (FREE)
2. Smart provider selection based on query complexity
3. Automatic fallback when providers fail

Perfect for production apps where cost matters!
"""

from llmswap import LLMClient
import time

def smart_assistant(question):
    """
    Automatically picks the most cost-effective option for each query
    """
    # Setup different cost tiers
    cached = LLMClient(cache_enabled=True)
    
    print(f"📝 Question: {question}")
    
    # Step 1: Check cache first (FREE)
    print("🔍 Checking cache...", end=" ")
    response = cached.query(question)
    
    if response.from_cache:
        print("✅ Found in cache (Cost: $0.00)")
        return response.content
    else:
        print(f"💰 API call made (Cost: ~$0.01)")
        return response.content

def main():
    print("🤖 Smart Cost Optimizer Demo")
    print("Watch how llmswap saves money automatically!\n")
    
    # Demo queries - some will be cached, some won't
    test_queries = [
        "What is Python?",              # First time - API call
        "What are Python data types?",  # New question - API call  
        "What is Python?",              # Repeat - cached (FREE!)
        "What are Python data types?",  # Repeat - cached (FREE!)
        "Explain Python functions",     # New question - API call
        "What is Python?",              # Again - cached (FREE!)
    ]
    
    total_queries = len(test_queries)
    api_calls = 0
    cached_calls = 0
    
    for i, question in enumerate(test_queries, 1):
        print(f"\n--- Query {i}/{total_queries} ---")
        
        # Track if this will be an API call or cache hit
        client = LLMClient(cache_enabled=True)
        response = client.query(question)
        
        if response.from_cache:
            cached_calls += 1
            print("✅ Served from cache (FREE)")
        else:
            api_calls += 1
            print("💳 API call made")
            
        print(f"Answer: {response.content[:100]}...")
        time.sleep(0.5)  # Small delay for demo effect
    
    # Show savings
    print(f"\n📊 Cost Analysis:")
    print(f"   API calls: {api_calls} × $0.01 = ${api_calls * 0.01:.3f}")
    print(f"   Cached calls: {cached_calls} × $0.00 = $0.000")
    print(f"   Total cost: ${api_calls * 0.01:.3f}")
    print(f"   Without caching: ${total_queries * 0.01:.3f}")
    
    if cached_calls > 0:
        savings_percent = (cached_calls / total_queries) * 100
        print(f"   💰 Savings: {savings_percent:.0f}%")
    
    print(f"\n🎉 In production, this scales to massive savings!")
    print(f"   FAQ bots: 90% cost reduction")
    print(f"   Documentation assistants: 80% savings") 
    print(f"   Repeated workflows: 85% cost cut")

if __name__ == "__main__":
    main()

==> llmswap-4.1.1.tar.gz: llmswap-4.1.1/examples/smart_cost_optimizer.py <==
"""
Smart Cost Optimizer - Save 50-90% on LLM API costs

This example shows how llmswap automatically optimizes costs by:
1. Using cache for repeated queries (FREE)
2. Smart provider selection based on query complexity
3. Automatic fallback when providers fail

Perfect for production apps where cost matters!
"""

from llmswap import LLMClient
import time

def smart_assistant(question):
    """
    Automatically picks the most cost-effective option for each query
    """
    # Setup different cost tiers
    cached = LLMClient(cache_enabled=True)
    
    print(f"Question: {question}")
    
    # Step 1: Check cache first (FREE)
    print("Checking cache...", end=" ")
    response = cached.query(question)
    
    if response.from_cache:
        print("Found in cache (Cost: $0.00)")
        return response.content
    else:
        print(f"API call made (Cost: ~$0.01)")
        return response.content

def main():
    print("Smart Cost Optimizer Demo")
    print("Watch how llmswap saves money automatically!\n")
    
    # Demo queries - some will be cached, some won't
    test_queries = [
        "What is Python?",              # First time - API call
        "What are Python data types?",  # New question - API call  
        "What is Python?",              # Repeat - cached (FREE!)
        "What are Python data types?",  # Repeat - cached (FREE!)
        "Explain Python functions",     # New question - API call
        "What is Python?",              # Again - cached (FREE!)
    ]
    
    total_queries = len(test_queries)
    api_calls = 0
    cached_calls = 0
    
    for i, question in enumerate(test_queries, 1):
        print(f"\n--- Query {i}/{total_queries} ---")
        
        # Track if this will be an API call or cache hit
        client = LLMClient(cache_enabled=True)
        response = client.query(question)
        
        if response.from_cache:
            cached_calls += 1
            print("Served from cache (FREE)")
        else:
            api_calls += 1
            print("API call made")
            
        print(f"Answer: {response.content[:100]}...")
        time.sleep(0.5)  # Small delay for demo effect
    
    # Show savings
    print(f"\nCost Analysis:")
    print(f"   API calls: {api_calls} × $0.01 = ${api_calls * 0.01:.3f}")
    print(f"   Cached calls: {cached_calls} × $0.00 = $0.000")
    print(f"   Total cost: ${api_calls * 0.01:.3f}")
    print(f"   Without caching: ${total_queries * 0.01:.3f}")
    
    if cached_calls > 0:
        savings_percent = (cached_calls / total_queries) * 100
        print(f"   Savings: {savings_percent:.0f}%")
    
    print(f"\nIn production, this scales to massive savings!")
    print(f"   FAQ bots: 90% cost reduction")
    print(f"   Documentation assistants: 80% savings") 
    print(f"   Repeated workflows: 85% cost cut")

if __name__ == "__main__":
    main()
